/*****************************************************************************/

/// \file       Handlers.c
/// \author     Frank Redlich
/// \date       12.11.2022
/// \brief      Interrupt Service Routines 
/// \details    Projekt:    Carrera_Decoder_PIC16F18424<br>
///             Prozessor:  Microchip PIC16F18424 <br>
///             Hardware:   DIY2, Decoder Mini
/// \version    1.01
///             durch die jetzt getrennten Pulslngen fr High und Low-Pulse, Anpassung in 
///             der Auswertung der Pulse  
/// \version    1.02
///             adptapted to PIC16F18424 
 
/******************************************************************************/



#include <xc.h>
#include "typedef.h"
#include "defines.h"
#include "ports.h"
#include "variable.h"


/**
 * Interrup Service Routinen
 * beim PIC16 gibt es nur einen einzigen Interrupt Vector
 * an Hand der Interrupt Flags der einzelnen Ressourcen muss abgefragt werden,
 * welcher Interrupt ausgelst wurde
 */
void __interrupt() INTERRUPT_InterruptManager (void)
{
    union Timer_Union   Timer1;                                                 // Timer 1 Register mit MSB und LSB

    S16                 Pulslaenge;                                             // gemessene Lnge des Pulses 
    
    U8                  Pulstyp;                                                // Art des Pulses, High, Low, Lnge

    
    
    
    


    //-------------------------------------------------------------------------- Timer 6 Interrupt       
    if (PIR4bits.TMR6IF == 1)
    {
        switch(Bitcounter)
        {
            // Aktiv Datenwort wird im Hauptprogramm nicht verwendet
            //case 8:
            //    Telegrammtyp = AKTIVDATENWORT;
            //    Telegramm_empfangen = JA;
            //    break;
            case 10:
                Telegrammtyp = REGLERDATENWORT;
                Telegramm_empfangen = JA;
                break;
            case 13:
                Telegrammtyp = PROGRAMMIERDATENWORT;
                Telegramm_empfangen = JA;
                break;
            default:                                                            // falsche Telegrammlnge
                DataBit[0] = 'X';                                               // Startbit ungltig machen fr erneuten Empfang
                break;
        }
        T6CONbits.TMR6ON = 0;                                                   // Timer 6 abschalten
        PIE4bits.TMR6IE = 0;                                                    // Interrupt wieder sperren
        PIR4bits.TMR6IF = 0;                                                    // Interruptflag lschen
        TMR6 = 0;                                                               // Timer 6 rcksetzen        
    }
    

    //-------------------------------------------------------------------------- Portpin Daten Interrupt
    
    if (IOCAFbits.IOCAF5 == 1)
    {        
        T1CONbits.TMR1ON = 0;                                                   // Timer 1 anhalten
        Timer1.Byte.LSB = TMR1L;
        Timer1.Byte.MSB = TMR1H;
        T1CONbits.TMR1ON = 1;                                                   // Timer 1 einschalten
        
        Pulslaenge = Timer1.Wort - Alter_Timer_Wert;                            // Pulslaenge = Neuer_Timer_Wert - Alter_Timer_Wert;        
        Alter_Timer_Wert = Timer1.Wort;                                         // Alter_Timer_Wert = Neuer_Timer_Wert;

        //----------------------------------------------------------------------   ermitteln, was das fr ein Puls war
        Pulstyp = PULS_UNBEKANNT;                                               // noch ist nicht klar was das war
#ifdef INPUT_INVERTED
        if (Pin_i_DATA == 1)                                                    // wenn jetzt High ist, dann war es ein:  
#else
        if (Pin_i_DATA == 0)                                                    // wenn jetzt Low ist, dann war es ein:      
#endif
        {                                                                       // Low Puls         
            if ((Pulslaenge > LOW_PULS_KURZ_MIN) && (Pulslaenge < LOW_PULS_KURZ_MAX))  
                Pulstyp = PULS_KURZ_LOW;
            if ((Pulslaenge > LOW_PULS_LANG_MIN) && (Pulslaenge < LOW_PULS_LANG_MAX))  
                Pulstyp = PULS_LANG_LOW;
            if ((Pulslaenge > PULS_START_MIN) && (Pulslaenge < PULS_START_MAX)) // Startbit 
                Pulstyp = PULS_START;
        }
        else                                                                    // wenn jetzt Low ist, dann war es ein High Puls 
        {
            if ((Pulslaenge > HIGH_PULS_KURZ_MIN) && (Pulslaenge < HIGH_PULS_KURZ_MAX))  
                Pulstyp = PULS_KURZ_HIGH;
            if ((Pulslaenge > HIGH_PULS_LANG_MIN) && (Pulslaenge < HIGH_PULS_LANG_MAX))  
                Pulstyp = PULS_LANG_HIGH;
        }        

        //----------------------------------------------------------------------  Auswertung je nach Puls 
        if (Telegramm_empfangen == NEIN)                                        // nur in den Empfagspuffer schreiben, wenn das letzte   
        {                                                                       // Telegramm schon abgeholt wurde      
            switch (Pulstyp)                                                        
            {
                case PULS_START:

                    Bitmitte = NEIN;
                    DataBit[0] = 'S';
                    Bitcounter = 1;

                    T6CONbits.TMR6ON = 0;                                       // Timer 6 ist schon abschaltet                
                    TMR6 = 0;                                                   // Timer 6 rcksetzen
                    PIR4bits.TMR6IF = 0;                                        // Interrupt Flag rcksetzen
                    PIE4bits.TMR6IE = 1;                                        // Interrupt wieder erlauben
                
                    T6CONbits.TMR6ON = 1;                                       // Timer 6 einschalten

                    
                    break;

                case PULS_KURZ_HIGH:
                case PULS_KURZ_LOW:
                
                    if (DataBit[0] == 'S')                                      // wenn die Startlcke da war
                    {   
                        if (Bitmitte == NEIN)                                   // ein kurzer Puls knnte die Bit-Mitte sein
                        {                                                       // oder das Bit-Ende 
                            Bitmitte = JA;
                            TMR6 = 0;                                           // Timer 6 rcksetzen
                        }
                        else
                        {
                            Bitmitte = NEIN;
                            if (Pulstyp == PULS_KURZ_LOW)
                                DataBit[Bitcounter] = 1;
                            else
                                DataBit[Bitcounter] = 0;
                    
                            if (Bitcounter < (MAX_DATABITS - 1))                            
                                Bitcounter++;

                            TMR6 = 0;                                           // Timer 6 rcksetzen
                        }            
                    }            
                    break;
                
                
                case PULS_LANG_HIGH:
                case PULS_LANG_LOW:
                
                    if (DataBit[0] == 'S')                                      // wenn die Startlcke da war
                    {
                        Bitmitte = NEIN;
                    
                        if (Pulstyp == PULS_LANG_LOW)
                            DataBit[Bitcounter] = 1;
                        else
                            DataBit[Bitcounter] = 0;

                        if (Bitcounter < (MAX_DATABITS - 1))                            
                            Bitcounter++;

                        TMR6 = 0;                                               // Timer 6 rcksetzen
                    }
                    break;
                
                default:                                                        // unbekannte Pulslnge / Fehler
                    DataBit[0] = 'X';                                           // Startbit ungltig machen fr erneutemn Empfang
                                        
                    break;
            } // Ende switch Pulstyp
        } // Ende das letzte Telegramm wurde schon im Hauptprogramm verarbeitet
    
        IOCAFbits.IOCAF5 = 0;                                                   // Interrupt Flag wieder lschen
    }
      
    

}

