/*****************************************************************************/

/// \file       main.c
/// \author     Frank Redlich
/// \date       12.11.2022
/// \brief      Digital Decoder fr Carrera
/// \details    Projekt:    Carrera_Decoder_PIC16F18424<br>
///             Prozessor:  Microchip PIC16F18424 <br>
///             Hardware:   DIY2, Decoder Mini
/// \version    1.01
///             Kombileuchte oder separate Rck- und Bremsleuchten 
/// \version    1.02
///             Aktivdatenwort wird ausgewertet
/// \version    1.03 by [MiZu]
///             It was possible to program car to different ID without power cycle.
///             This worked by putting the car in ready for power cycle mode with one ID
///             and then set the ID with another controller. Main change in decoder.c
///             Small adjustment here.
/// \version    1.04
///             adapted to PIC16F18F24

/******************************************************************************/


#include <xc.h>
#include "version.h"
#include "typedef.h"
#include "defines.h"
#include "variable.h"
#include "ports.h"
#include "init.h"
#include "decoder.h"
#include "eeprom.h"
#include "delays.h"

 

// CONFIG1
#pragma config FEXTOSC = OFF        // External Oscillator mode selection bits->Oscillator not enabled
#pragma config RSTOSC = HFINT1      // Power-up default value for COSC bits->HFINTOSC (1MHz)
#pragma config CLKOUTEN = OFF       // Clock Out Enable bit->CLKOUT function is disabled; i/o or oscillator function on OSC2
#pragma config CSWEN = ON           // Clock Switch Enable bit->Writing to NOSC and NDIV is allowed
#pragma config FCMEN = ON           // Fail-Safe Clock Monitor Enable bit->FSCM timer enabled

// CONFIG2
#pragma config MCLRE = ON           // Master Clear Enable bit->MCLR pin is Master Clear function
#pragma config PWRTS = OFF          // Power-up Timer Enable bit->PWRT disabled
#pragma config LPBOREN = OFF        // Low-Power BOR enable bit->ULPBOR disabled
#pragma config BOREN = ON           // Brown-out reset enable bits->Brown-out Reset Enabled, SBOREN bit is ignored
#pragma config BORV = LO            // Brown-out Reset Voltage Selection->Brown-out Reset Voltage (VBOR) set to 2.45V
#pragma config ZCDDIS = OFF         // Zero-cross detect disable->Zero-cross detect circuit is disabled at POR.
#pragma config PPS1WAY = ON         // Peripheral Pin Select one-way control->The PPSLOCK bit can be cleared and set only once in software
#pragma config STVREN = ON          // Stack Overflow/Underflow Reset Enable bit->Stack Overflow or Underflow will cause a reset

// CONFIG3
#pragma config WDTCPS = WDTCPS_31   // WDT Period Select bits->Divider ratio 1:65536; software control of WDTPS
//#pragma config WDTE = OFF         // WDT operating mode->WDT Disabled, SWDTEN is ignored
//#pragma config WDTE = ON    		// WDT operating mode->WDT enabled regardless of sleep; SWDTEN ignored
#pragma config WDTE = SWDTEN        // WDT operating mode->WDT enabled/disabled by SWDTEN bit in WDTCON0

#pragma config WDTCWS = WDTCWS_7    // WDT Window Select bits->window always open (100%); software control; keyed access not required
#pragma config WDTCCS = SC          // WDT input clock selector->Software Control

// CONFIG4
#pragma config BBSIZE = BB512       // Boot Block Size Selection bits->512 words boot block size
#pragma config BBEN = OFF           // Boot Block Enable bit->Boot Block disabled
#pragma config SAFEN = OFF          // SAF Enable bit->SAF disabled
#pragma config WRTAPP = OFF         // Application Block Write Protection bit->Application Block not write protected
#pragma config WRTB = OFF           // Boot Block Write Protection bit->Boot Block not write protected
#pragma config WRTC = OFF           // Configuration Register Write Protection bit->Configuration Register not write protected
#pragma config WRTD = OFF           // Data EEPROM write protection bit->Data EEPROM NOT write protected
#pragma config WRTSAF = OFF         // Storage Area Flash Write Protection bit->SAF not write protected
#pragma config LVP = ON             // Low Voltage Programming Enable bit->Low Voltage programming enabled. MCLR/Vpp pin function is MCLR.

// CONFIG5
#pragma config CP = OFF             // UserNVM Program memory code protection bit->UserNVM code protection disabled



void main()
{
    U8      address;            
       
    
    Initialisierung();                                                          // Ports, Timer etc. 
    
    Init_EEPROM_Defaultwerte();
    
    WDTCON0 = 0b010001;                                                         // switch on watchdog, WDTPS[4:0] = 1:8192 (Interval 256 ms nominal) 
    CLRWDT();                                                                   // clear Watchdog     
    
    //--------------------------------------------------------------------------

    Licht_schalten();                                                           // je nach Wert im EEPROM Licht ein oder aus
    IR_LED_einschalten();                                                       // mit der Frequenz entsprechend der Car Adresse

    //--------------------------------------------------------------------------
    //if ((EE_read(EE_ADR_PROGRAMMIERMODUS)) == JA)                               // Auto Adresse Programmiermodus JA/NEIN
    if ((EE_read(EE_ADR_PROGRAMMIERMODUS)) != NEIN)                             // Auto Adresse Programmiermodus NEIN, [MiZu] oder Controller ID, die Programmierung angefordert hat        
        Adr_Programmier_timeout = PROGRAMMIER_TIMEOUT;                          // ca. 3 sec Programmiermglichkeit fr Adresse
    else
        Adr_Programmier_timeout = 0;
    
    
   
    
    while(1)
    {       
        if (Telegramm_empfangen == JA)                                          // alle 7,5 msec
        {            
            CLRWDT();                                                           // clear Watchdog 

            Telegramm_empfangen = NEIN;

            DataBit[0] = 'X';                                                   // Startbit ungltig machen fr erneutemn Empfang
            Timer_Counter++;                                                    // allgemeiner Timer 7,5 ms
            
            //...................................................
            
            if (Licht_Sperr_Timer > 0)                                          // auf Null zhlen
                Licht_Sperr_Timer--;                                            // jetzt darf Licht geschaltet werden

            eigene_Adresse = (U8)(EE_read(EE_ADR_AUTO_ADRESSE) & 0x07);         // wird in dieser Routine mehrfach bentigt  
                    
            switch (Telegrammtyp) 
            {
                case AKTIVDATENWORT:     //.....................................   Aktivdatenwort
                    
                    if (CU_Daten.Adresse < 6)                                   // nur normale Autos, kein Ghostcar
                    {
                        if (DataBit[eigene_Adresse + 1] == 0)                   // wenn der eigene Regler nicht gedrckt
                            CCPR1L = 0;                                         // Motor ausschalten
                    }                                         
                    break;

                case REGLERDATENWORT: //........................................  Regler oder Ghostcar Datenwort
                    
                    Reglerdatenwort_decodieren();

                    if (
                            (Ghostcar_Doppelklick_timeout == 0) &&              // nur wenn nicht auf Doppelklick fr Ghostcar gewartet wird
                            (CU_Daten.Adresse < 6)
                        )
                        Count_Spurwechseltaste(CU_Daten.Adresse);

                    if (CU_Daten.Adresse == eigene_Adresse)
                    {
                        Motor_und_Bremse_ansteuern(CU_Daten.Reglerwert);        // Motor in Abhngigkeit der Reglerstellung
                        if (Tank_leer == JA)                                    // wenn Tank leer dann blinken ==> kommt aus einem Programmierdatenwort
                            Tank_leer_Blinken = JA;
                            
                        Eigene_Spurwechseltaste_auswerten();                    // Schalten von Licht bzw. Wechsel in Ghostcarprogrammierung

                        if (Adr_warte_auf_Reset_timeout > 0)                    // wenn 2x Spurwechseltaste gedrckt gedrckt aber noch nicht angehoben
                        {
                            if (CU_Daten.Reglerwert > 0)                        // und wenn das Auto fhrt
                                Adr_warte_auf_Reset_timeout = 1;                // dann beim nchstenTimerclick den Programmiermodus sofort verlassen
                        }
                    }

                    if (eigene_Adresse == GHOSTCAR_ADRESSE)                     // Ghostcarbetrieb
                    {
                        v_Ghostcar = (U8)(EE_read(EE_ADR_GHOST_GESCHWINDIGKEIT) & 0x0F);
                                
                        if (CU_Daten.Adresse == PACE_UND_GHOSTCAR_DW_ADRESSE)
                        {
                            CU_Daten.KFR = DataBit[4];                          // Keine Freigabe Bit
                            CU_Daten.FR = DataBit[6];                           // Freigabe Bit
                            if ((CU_Daten.KFR == 0) && (CU_Daten.FR == 1))
                                Motor_und_Bremse_ansteuern(v_Ghostcar);         // programmierte Geschwindigkeit
                            if ((CU_Daten.KFR == 1) && (CU_Daten.FR == 0))
                                Motor_und_Bremse_ansteuern(0);                  // Motor aus, z.B. in Startphase
                        }
                    }
                    
                    break;

                case PROGRAMMIERDATENWORT: //.................................  Programmierdatenwort
                    Programmierdatenwort_decodieren();
                    if (CU_Prg_Daten.Adresse == eigene_Adresse)
                    {
                        if (                                                    // das wid immer 2 x bertragen
                                (CU_Prg_Daten.Parameter == PRG_MAX_GESCHWINDIGKEIT) ||
                                (CU_Prg_Daten.Parameter == PRG_BREMSWIRKUNG) //||
                                //(CU_Prg_Daten.Parameter == TANK_INHALT)
                            )
                        {
                            //. . . . . . . . . . . . . . . . . . . . . . . . .    max. Geschwindigkeit oder Bremswirkung
                            if (                                                // Wenn das Parametrierwort schon mal gesendet wurde
                                    (CU_Prg_Daten.Parameter == CU_Prg_Daten.Parameter_alt) &&
                                    (CU_Prg_Daten.Wert == CU_Prg_Daten.Wert_alt)
                               )

                            {
                                Parameter_verarbeiten();                        // max. Gewschindigkeit und Bremswirkung speichern
                                CU_Prg_Daten.Parameter_alt = PRG_NORMALBETRIEB;
                                CU_Prg_Daten.Wert_alt = 0;
                            }
                            else
                            {
                                CU_Prg_Daten.Parameter_alt = CU_Prg_Daten.Parameter;// merken was gerade gesendet wurde
                                CU_Prg_Daten.Wert_alt = CU_Prg_Daten.Wert;
                            }
                        }
                        else
                        {
                            if (CU_Prg_Daten.Parameter == PRG_NORMALBETRIEB)
                            {
                                if ((CU_Prg_Daten.Wert & 0x08) != 0)
                                    Tank_leer = JA;
                                else
                                    Tank_leer = NEIN;
                            }
                        }

                    }
                    break;
                                       
            }

            //----------------------------------------------------------------  ab hier alle ca. 100msec
            //if (Timer_Counter > 13)                                           // wenn alle Telegramme detektiert werden
            if (Timer_Counter > 10)                                             // wenn die Aktivdatenwrter nicht detektiert werden                            
            {
                Timer_Counter = 0;
                
                
                
                //..............................................................
                if (Adr_warte_auf_Reset_timeout > 0)                            // Programmiersequenz ist gestartet aber es ist noch kein Neustart gewesen
                {
                    Adr_warte_auf_Reset_timeout--;
                    if (Adr_warte_auf_Reset_timeout == 0)                       // wenn 2 Sekunden kein Neuart
                    {
                        Programmiermodus = NEIN;                                // dann Programmiermodus verlassen
                        EE_write_verified (NEIN, EE_ADR_PROGRAMMIERMODUS);
                    }
                }

                //..............................................................
                if (Adr_Programmier_timeout > 0)                                // Programmiermodus nach Neustart nach 2 Sekunden abbrechen
                {
                    Pin_o_LICHT ^= 1;
                    Adr_Programmier_timeout--;                                  // Timeout fr Adresssierung Auto Adresse
                    if (Adr_Programmier_timeout == 0)
                    {
                        Programmiermodus = NEIN;                                // dann Programmiermodus verlassen                        
                        EE_write_verified (NEIN, EE_ADR_PROGRAMMIERMODUS);
                        Licht_schalten();                                       // je nach Wert im EEPROM Licht ein oder aus
                    }
                }

                //..............................................................
                if (Ghostcar_Doppelklick_timeout > 0)                           // Zeitfenster fr Doppelklick nach Adr.-Programmierung
                    Ghostcar_Doppelklick_timeout--;                             // beenden

                
                //..............................................................
                if (Bremslicht_timer > 0)
                {
                    Bremslicht_timer--;
                    
                    Pin_o_BREMSLICHT = 1;
                }
                else
                {
                    Pin_o_BREMSLICHT = 0;                                       // Bremslicht aus 
                }
               
                //..............................................................
                for (address = 0; address < MAX_REGLER; address++)              // fr alle Regler
                {
                    if (Spurwechseltaste.Reset_Timer[address] > 0)              // wenn 2 Sekunden keine Taste gedrckt, dann
                    {                                                           // den Counter lschen
                        Spurwechseltaste.Reset_Timer[address]--;                // d.h. zum Programmieren muss die Taste 2 x innerhalb
                        if (Spurwechseltaste.Reset_Timer[address] == 0)         // von 2 Sekunden gedrckt werden
                        {
                            Spurwechseltaste.Counter[address] = 0;
                        }
                    }
                }


                //..............................................................
                
                Ghostcar_Programmiermodus = EE_read(EE_ADR_GHOST_PROGRAMMIERMODUS);
                if (
                        (Ghostcar_Programmiermodus == JA) ||
                        (Tank_leer_Blinken == JA)
                    )
                                                                                // Im Ghostcar Programmiermodus (Geschwindigkeit einstellen)
                                                                                // und wenn der Tank leer ist, dann
                {                                                               // langsam blinken
                    Pin_o_LICHT ^= 1;  
                    
                    if ((Tank_leer_Blinken == JA) && (Tank_leer == NEIN))       // wenn Tank wieder voll, dann Licht schalten wie es war
                    {
                        Tank_leer_Blinken = NEIN;                               // aufhren mit Blinken
                        Licht_schalten();
                    }
                }
            } // Ende von 131 ms vorbei
        } // Ende von es wurde ein Telegramm empfangen
    } // Ende Hauptschleife
}	





